@@ -19,7 +19,7 @@
 
 #define LABEL_DATA "gnome-keybinding-properties-label"
 #define KEY_THEME_KEY "/desktop/gnome/interface/gtk_key_theme"
-#define MAX_ELEMENTS_BEFORE_SCROLLING 8
+#define MAX_ELEMENTS_BEFORE_SCROLLING 10
 
 #if defined(__powerpc__) && defined (__linux__)
 #define USE_FBLEVEL
@@ -41,32 +41,36 @@ typedef struct
 
 const KeyListEntry desktop_key_list[] =
 {
+  { "/apps/gnome_settings_daemon/keybindings/help", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/power", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/sleep", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/screensaver", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/home", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/search", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/email", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/www", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/panel_run_dialog", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/panel_main_menu", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/run_command_screenshot", ALWAYS_VISIBLE, 0 },
   { "/apps/metacity/global_keybindings/run_command_window_screenshot", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/volume_mute", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/volume_down", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/volume_up", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/power", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/eject", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/home", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/search", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/email", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/sleep", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/screensaver", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/help", ALWAYS_VISIBLE, 0 },
-  { "/apps/gnome_settings_daemon/keybindings/www", ALWAYS_VISIBLE, 0 },
 #ifdef USE_FBLEVEL
   { "/apps/gnome_settings_daemon/keybindings/brightness_down", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/brightness_up", ALWAYS_VISIBLE, 0 },
 #endif
+  { NULL }
+};
+const KeyListEntry sounds_key_list[] =
+{
+  { "/apps/gnome_settings_daemon/keybindings/volume_mute", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/volume_down", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/volume_up", ALWAYS_VISIBLE, 0 },
   /* Other ones that need keysyms bindings */
   { "/apps/gnome_settings_daemon/keybindings/play", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/pause", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/stop", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/previous", ALWAYS_VISIBLE, 0 },
   { "/apps/gnome_settings_daemon/keybindings/next", ALWAYS_VISIBLE, 0 },
+  { "/apps/gnome_settings_daemon/keybindings/eject", ALWAYS_VISIBLE, 0 },
   { NULL }
 };
 
@@ -594,6 +598,7 @@ reload_key_entries (gpointer wm_name, GladeXML *dialog)
   clear_old_model (dialog, WID ("shortcut_treeview"));
   
   append_keys_to_tree (dialog, _("Desktop"), desktop_key_list);
+  append_keys_to_tree (dialog, _("Sound"), sounds_key_list);
   
   if (strcmp((char *) wm_name, WM_COMMON_METACITY) == 0)
     {
@@ -697,19 +702,18 @@ accel_edited_callback (GtkCellRendererText   *cell,
   GError *err = NULL;
   char *str;
 
-  model = get_real_model (view);
+  model = gtk_tree_view_get_model (view);
   gtk_tree_model_get_iter (model, &iter, path);
+  gtk_tree_path_free (path);
   gtk_tree_model_get (model, &iter,
 		      KEYENTRY_COLUMN, &key_entry,
 		      -1);
 
   /* sanity check */
   if (key_entry == NULL)
-    {
-      gtk_tree_path_free (path);
-      return;
-    }
+    return;
 
+  model = get_real_model (view);
   tmp_key.model  = model;
   tmp_key.keyval = keyval;
   tmp_key.keycode = keycode;
@@ -745,7 +749,6 @@ accel_edited_callback (GtkCellRendererText   *cell,
       /* set it back to its previous value. */
       egg_cell_renderer_keys_set_accelerator (EGG_CELL_RENDERER_KEYS (cell),
 					      key_entry->keyval, key_entry->keycode, key_entry->mask);
-      gtk_tree_path_free (path);
       return;
     }
 
@@ -773,8 +776,6 @@ accel_edited_callback (GtkCellRendererText   *cell,
       g_error_free (err);
       key_entry->editable = FALSE;
     }
-  
-  gtk_tree_path_free (path);
 }
 
 static void
@@ -789,18 +790,16 @@ accel_cleared_callback (GtkCellRendererText *cell,
   GError *err = NULL;
   GtkTreeModel *model;
 
-  model = get_real_model (view);
+  model = gtk_tree_view_get_model (view);
   gtk_tree_model_get_iter (model, &iter, path);
+  gtk_tree_path_free (path);
   gtk_tree_model_get (model, &iter,
 		      KEYENTRY_COLUMN, &key_entry,
 		      -1);
 
   /* sanity check */
   if (key_entry == NULL)
-    {
-      gtk_tree_path_free (path);
-      return;
-    }
+    return;
 
   /* Unset the key */
   gconf_client_set_string (gconf_client_get_default(),
@@ -823,7 +822,6 @@ accel_cleared_callback (GtkCellRendererText *cell,
       g_error_free (err);
       key_entry->editable = FALSE;
     }
-  
 }
 
 static void
@@ -928,7 +926,7 @@ cb_dialog_response (GtkWidget *widget, gint response_id, gpointer data)
 {
 	if (response_id == GTK_RESPONSE_HELP)
 		capplet_help (GTK_WINDOW (widget),
-			      "wgoscustdesk.xml",
+			      "user-guide.xml",
 			      "goscustdesk-39");
 	else
 		gtk_main_quit ();
